<?php

	/**
	 * Klasse, die ein Produkt behandelt
	 */
	class wpsg_product extends wpsg_model
	{

		const MULTIPLE_ONE_MULTI = 0; // Nur einmal mit beliebiger Menge (Standard)
		const MULTIPLE_ONE_ONE = 4; // Nur einmal mit Menge 1
		const MULTIPLE_MULTI_MULTI = 1; // Mehrfach mit beliebiger Menge
		const MULTIPLE_MULTI_ONE = 2; // Mehrfach mit Menge 1

		private $loadedData = array(); /* Enthält die Daten, die über den ShopController und die Module geladen / erweitert wurden */

		/**
		 * Lädt die Daten des Produkts
		 */
		public function load($product_id, $loadedData = false)
		{

			parent::load($product_id);
							
			$this->data = $this->db->fetchRow("SELECT P.* FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE P.`id` = '".wpsg_q($product_id)."' ");

			if ($this->shop->isOtherLang())
			{
			
				$trans_produkt = $this->db->fetchRow("
					SELECT
						P.`id`, P.`name`, P.`beschreibung`, P.`detailname`
					FROM
						`".WPSG_TBL_PRODUCTS."` AS P
					WHERE
						P.`lang_locale` = '".wpsg_q($this->shop->getCurrentLanguage())."' AND
						P.`lang_parent` = '".wpsg_q($product_id)."'
				");
			
				if ($trans_produkt['id'] > 0)
				{
						
					$this->data['name'] = $trans_produkt['name'];
					$this->data['beschreibung'] = $trans_produkt['beschreibung'];
					$this->data['detailname'] = $trans_produkt['detailname'];
						
				}
				
			}
			
			if ($this->data['id'] != $product_id) $this->shop->throwErrorCode('100_5', array('Gesuchte ID:'.$product_id, 'Gefundene ID:'.$this->data['id']));

			if (wpsg_isSizedArray($loadedData)) $this->loadedData = $loadedData;
			else $this->loadedData = $this->shop->loadProduktArray($this->data['id']);

			return true;
				
		} // public function load($product_id)
		
		public function delete()
		{
						 
			$this->db->UpdateQuery(WPSG_TBL_PRODUCTS, array(
				'deleted' => '1'
			), "`id` = '".wpsg_q($this->id)."'");
				
			$this->shop->callMods('produkt_del', array($this->id));
				
			// Übersetzung löschen
			$this->db->UpdateQuerY(WPSG_TBL_PRODUCTS, array(
				'deleted' => '1'
			), "`lang_parent` = '".wpsg_q($this->id)."'");
						
		} // public function delete()
		
		/**
		 * Setzt einen Wert in der META Tabelle
		 */
		public function setMeta($meta_key, $meta_value)
		{
				
			$nExistsID = $this->db->fetchOne("SELECT M.`id` FROM `".WPSG_TBL_META."` AS M WHERE M.`meta_table` = 'WPSG_TBL_PRODUCTS' AND M.`meta_key` = '".wpsg_q($meta_key)."' AND `target_id` = '".wpsg_q($this->id)."' ");
				
			if (wpsg_isSizedInt($nExistsID))
			{
		
				$this->db->UpdateQuery(WPSG_TBL_META, array(
					'meta_value' => wpsg_q($meta_value)
				), " `id` = '".wpsg_q($nExistsID)."' ");
		
			}
			else
			{
		
				$this->db->ImportQuery(WPSG_TBL_META, array(
					'target_id' => wpsg_q($this->id),
					'meta_table' => "WPSG_TBL_PRODUCTS",
					'meta_key' => wpsg_q($meta_key),
					'meta_value' => wpsg_q($meta_value)
				));
		
			}
				
		} // public function setMeta($meta_key, $meta_value)
		
		/**
		 * Gibt einen META Value zurück
		 */
		public function getMeta($meta_key)
		{
				
			$meta_value = $this->db->fetchOne("SELECT M.`meta_value` FROM `".WPSG_TBL_META."` AS M WHERE M.`meta_table` = 'WPSG_TBL_PRODUCTS' AND M.`meta_key` = '".wpsg_q($meta_key)."' AND `target_id` = '".wpsg_q($this->id)."' ");
				
			return $meta_value;
				
		} // public function getMeta($meta_key)

		/**
		 * Gibt die Produktbeschreibung zurück
		 */
		public function getShortDescription()
		{

			return $this->data['beschreibung'];

		} // public function getShortDescription()

		/**
		 * Gibt den Namen des Produktes zurück
		 */
		public function getProductName($detailname = false)
		{

			$strDetailname = $this->detailname;

			if (wpsg_isSizedString($strDetailname) && $detailname === true)
			{
				
				$strName = $strDetailname;
				
			}
			else
			{
			
				$strName = $this->name;
				
			}
			
			if (strpos($strName, '^') !== false)
			{

				$strName = preg_replace('/\^/', '<span class="wpsg_upper">', $strName).'</span>';
				
			}
			
			return $strName;
			
		} // public function getProductName()
		
		/**
		 * Gibt die ProduktURL zurück
		 * Siehe auch wpsg_ShopController->getProduktLink (Macht das gleiche, aber beachtet den Basket von welcher URL es hinzugefügt wurde da URL nicht eindeutig)
		 */
		public function getProductURL()
		{
			
			$url = false;
			
			$this->shop->callMods('getProduktlink', array($this->id, &$url));
			
			if ($url === false)
			{
				
				$partikel = $this->partikel;
			
				if (wpsg_isSizedInt($partikel))
				{
					
					return get_permalink($this->partikel);
					
				}
				else
				{
						
					$basket_url = $this->shop->getURL(wpsg_ShopController::URL_BASKET);
						
					if (strpos($basket_url, "?") > 0)
					{
						
						$basket_url .= "&wpsg_action=showProdukt&produkt_id=".$this->id;
						
					}
					else
					{
						
						$basket_url .= "?wpsg_action=showProdukt&produkt_id=".$this->id;
						
					}
											
					return $basket_url;
						
				}
			
			}
			else
			{
			
				return $url;
			
			}
			
		} // public function getProductURL()
		 
		/**
		 * Gibt den MwSt. Wert für das Frontend zurück
		 */
		public function getFrontendTaxValue()
		{
			
			return 19.00;
			
		} // public function getFrontendTaxValue()
		
		/**
		 * Gibt den Alten Preis für das Frontend zurück
		 */
		public function getOldPrice($taxView = false)
		{

			$oldprice = wpsg_tf($this->oldprice);

			if ($oldprice > 0) return $this->calculateTaxViewPrice($oldprice, $taxView);
			else return false;
			
		}
		
		public function getMinPrice()
		{
			
			return $this->getPrice();
			
		}
		
		/**
		 * Gibt den Preis für das Frontend zurück
		 */
		public function getPrice($taxView = false)
		{

			if ($taxView === false)
			{

				if (is_admin()) $taxView = $this->shop->getBackendTaxview();
				else $taxView = $this->shop->getFrontendTaxview();

			}
 
			if ($taxView === WPSG_BRUTTO)
			{

				return $this->loadedData['preis_brutto'];

			}
			else
			{

				return $this->loadedData['preis_netto'];

			}

		}
		
		/**
		 * Gibt true zurück, wenn die Zahlungsarten für das Produkt eingeschränkt sind
		 * @return bool 
		 */
		public function hasLimitedPayment()
		{
			
			$arAllowedPayment = explode(',', $this->allowedpayments);
			$arAllowedPayment = wpsg_trim($arAllowedPayment, array('', '0'));
			
			if (wpsg_isSizedArray($arAllowedPayment))
			{
				
				return true;
				
			}
			
		} // public function hasLimitedPayment()
				
		/**
		 * Gibt einen Array mit IDs von Zahlungsarten zurück, die für dieses Produkt erlaubt sind
		 * Vorsicht! Wenn alle erlaubt sind gibt es einen leeren Array zurück, deshalb mit hasLimitedPayment vorher prüfen
		 * @return array
		 */
		public function getAllowedPayment()
		{
			
			if (!$this->hasLimitedPayment()) return array();
			else
			{
				
				$arPayment = wpsg_trim((array)explode(',', $this->allowedpayments), '0');
				
				return $arPayment;
				
			}
			
		} // public function getAllowedPayment()
		
		/**
		 * Gibt true zurück, wenn die Versandarten für das Produkt eingeschränkt sind
		 */
		public function hasLimitedShipping()
		{
				
			$arAllowedShipping = explode(',', $this->allowedshipping);
			$arAllowedShipping = wpsg_trim($arAllowedShipping, array('', '0'));
			
			if (wpsg_isSizedArray($arAllowedShipping))
			{
		
				return true;
		
			}
			
			return false;
				
		} // public function hasLimitedShipping()
		
		/**
		 * Gibt einen Array mit IDs von Versandarten zurück, die für dieses Produkt erlaubt sind
		 * Vorsicht! Wenn alle erlaubt sind gibt es einen leeren Array zurück, deshalb mit hasLimitedPayment vorher prüfen
		 * @return array
		 */
		public function getAllowedShipping()
		{
			
			if (!$this->hasLimitedShipping()) return array();
			else
			{
				
				$arShipping = wpsg_trim((array)explode(',', $this->allowedshipping), '0');
				
				return $arShipping;
				
			}
			
		} // public function getAllowedShipping()

		/* Private Funktionen */

		private function calculateTaxViewPrice($price, $taxView)
		{

			if ($this->shop->getBackendTaxview() === WPSG_NETTO)
			{

				$price_netto = $price;
				$price_brutto = wpsg_calculatePreis($price_netto, WPSG_BRUTTO, $this->shop->getCalcTaxValue($this->mwst_key));

			}
			else
			{

				$price_brutto = $price;
				$price_netto = wpsg_calculatePreis($price_brutto, WPSG_NETTO, $this->shop->getCalcTaxValue($this->mwst_key));

			}

			if ($taxView === false)
			{

				if (is_admin()) $taxView = $this->shop->getBackendTaxview();
				else $taxView = $this->shop->getFrontendTaxview();

			}

			if ($taxView === WPSG_NETTO) return $price_netto;
			else return $price_brutto;

		}
		
		/* Statische Funktionen */
		
		public static function findMaxPrice($arFilter = array())
		{
			
			list($strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);
				
			$strQuery = "
				SELECT
					MAX(P.`preis`)
				FROM
					`".WPSG_TBL_PRODUCTS."` AS P
						".$strQueryJOIN."
				WHERE
					1
					".$strQueryWHERE."
				HAVING
					1
					".$strQueryHAVING."
			";
				
			return $GLOBALS['wpsg_db']->fetchOne($strQuery);
			
		}
		
		public static function findMinPrice($arFilter = array())
		{
		
			list($strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);
			
			$strQuery = "
				SELECT
					MIN(P.`preis`)
				FROM
					`".WPSG_TBL_PRODUCTS."` AS P
						".$strQueryJOIN."
				WHERE
					1
					".$strQueryWHERE."				
				HAVING
					1
					".$strQueryHAVING."
			";
					 	
			return $GLOBALS['wpsg_db']->fetchOne($strQuery);
			
		} // public static function getMinPrice($arFilter = array())
				
		public static function find($arFilter = array(), $load = true)
		{
		
			list($strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);
		
			$strQuery = "
				SELECT
					P.`id`
				FROM
					`".WPSG_TBL_PRODUCTS."` AS P
						".$strQueryJOIN."
				WHERE
					1
					".$strQueryWHERE."
				GROUP BY
					P.`id`
				HAVING
					1
					".$strQueryHAVING."				
				ORDER BY
					".$strQueryORDER."				
			";
			 
			$arID = $GLOBALS['wpsg_db']->fetchAssocField($strQuery);
			$arReturn = array();
		
			foreach ($arID as $id)
			{
		
				if ($load === true) $arReturn[$id] = self::getInstance($id);
				else $arReturn[$id] = $id;
		
			}
		
			return $arReturn;
		
		} // public function find($arQuery = array())
		
		public static function getQueryParts($arFilter = array())
		{
		  
			$strQueryWHERE = "";
			$strQueryJOIN = "";
			$strQueryHAVING = "";
			$strQueryORDER = "";
			
			if (wpsg_isSizedString($arFilter['order_col'])) $strQueryORDER .= " ".$arFilter['order_col']." ";
			if (wpsg_isSizedString($arFilter['order_direction'])) $strQueryORDER .= $arFilter['order_direction'];			
			if (!wpsg_isSizedString($strQueryORDER)) $strQueryORDER = " P.`cdate` ASC ";
			
			if (wpsg_isSizedArray($arFilter['product_ids'])) $strQueryWHERE .= " AND P.`id` IN (".wpsg_q(implode(',', $arFilter['product_ids'])).") ";
			if (wpsg_isSizedArray($arFilter['productgroup_ids'])) $strQueryWHERE .= " AND P.`pgruppe` IN (".wpsg_q(implode(',', $arFilter['productgroup_ids'])).") ";
			if (wpsg_isSizedString($arFilter['price_min'])) $strQueryWHERE .= " AND P.`preis` >= '".wpsg_q($arFilter['price_min'])."' ";
			if (wpsg_isSizedString($arFilter['price_max'])) $strQueryWHERE .= " AND P.`preis` <= '".wpsg_q($arFilter['price_max'])."' ";
			
			if (wpsg_isSizedArray($arFilter['cat_ids']) && $GLOBALS['wpsg_sc']->hasMod('wpsg_mod_produktartikel'))
			{
				
				$strQueryJOIN .= " LEFT JOIN `".wpsg_q($GLOBALS['wpsg_sc']->prefix.'posts')."` AS POST ON (POST.`wpsg_produkt_id` = P.`id`) ";
				$strQueryJOIN .= " LEFT JOIN `".wpsg_q($GLOBALS['wpsg_sc']->prefix.'term_relationships')."` AS PCAT ON (POST.`ID` = PCAT.`object_id`) ";
			
				$strQueryWHERE .= " AND PCAT.`term_taxonomy_id` IN (".wpsg_q(implode(',', $arFilter['cat_ids'])).") ";
				
			}
			
			if (wpsg_isSizedArray($arFilter['variants']))
			{
								
				foreach (wpsg_trim($arFilter['variants']) as $variant_id => $variation_id) 
				{
									
					$in = wpsg_q($variant_id);
					$strQueryJOIN .= " RIGHT JOIN `".WPSG_TBL_PRODUCTS_VARIATION."` AS PVI".$in." ON (PVI".$in.".`product_id` = P.`id` AND PVI".$in.".`variation_id` IN (".wpsg_q($variation_id).") AND PVI".$in.".`active` = '1') ";
										
				}
				 				
			}
			
			if (wpsg_isSizedArray($arFilter['attributs']))
			{
				
				foreach ($arFilter['attributs'] as $attribut_id => $attribut_value)
				{
					
					$in = wpsg_q($attribut_id);
					$arValues = wpsg_trim(explode(',', $attribut_value));
					 
					$strQueryJOIN_ATTRIBUTE_ON  = " ( 0 ";
					foreach ($arValues as $k => $value) $strQueryJOIN_ATTRIBUTE_ON .= " OR PA".$in.".`value` = '".wpsg_q($value)."' ";
					$strQueryJOIN_ATTRIBUTE_ON .= " ) ";
					
					$strQueryJOIN .= " RIGHT JOIN `".WPSG_TBL_PRODUCTS_AT."` AS PA".$in." ON (PA".$in.".`p_id` = P.`id` AND PA".$in.".`a_id` = '".wpsg_q($attribut_id)."' AND ".$strQueryJOIN_ATTRIBUTE_ON.") ";
					
				}
				 
			}
			
			$strQueryWHERE .= " AND P.`deleted` != '1' ";
			
			return array($strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER);
		
		} // public function getQueryParts($arFilter = array())
		
	} // class wpsg_product extends wpsg_model

?>